'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNetworkInterface = createNetworkInterface;

var _ramda = require('ramda');

var _networkInterface = require('apollo-client/transport/networkInterface');

var _phoenix = require('phoenix');

function applyWares(ctx, wares) {
  return new Promise(function (resolve, reject) {
    var queue = function queue(funcs, scope) {
      var next = function next() {
        if (funcs.length > 0) {
          var f = funcs.shift();
          f && (f.applyMiddleware || f.applyAfterware).apply(scope, [ctx, next]);
        } else {
          resolve(ctx);
        }
      };
      next();
    };
    queue(wares.slice());
  });
}

function executeQuery(sockets, context) {
  var request = context.request,
      options = context.options;
  var uri = options.uri,
      channel = options.channel;

  var Socket = options.Socket || _phoenix.Socket;

  if (!uri) throw "Missing options.uri";
  if (!channel) throw "Missing options.channel";
  if (!channel.topic) throw "Missing options.channel.topic";

  if (options.logger === true) {
    options.logger = function (kind, msg, data) {
      return console.log('Apollo websocket ' + kind + ': ' + msg, data);
    };
  }

  if (!sockets[uri]) {
    if (typeof Socket.constructor === 'function') {
      sockets[uri] = {
        conn: new Socket(uri, options),
        channels: {}
      };
    } else {
      sockets[uri] = {
        conn: Socket(),
        channels: {}
      };
    }
  }

  var socket = sockets[uri];
  var chan = socket.channels[channel.topic];

  function joinChannel(resolve) {
    if (!socket.conn.isConnected()) {
      return socket.conn.connect();
    }
    if (!chan.conn) {
      chan.conn = socket.conn.channel(channel.topic, channel.params || {});
    }
    if (!chan.conn.isJoined() && !chan.conn.isJoining()) {
      chan.conn.join().receive("ok", function (_) {
        var queue = chan.queue;
        chan.queue = [];
        (0, _ramda.map)(performQuery, queue);
      }).receive('error', function (err) {
        chan.conn.leave();
        chan.conn = null;
        chan.queue.pop();
        resolve(err);
      });
    }
  }

  function performQuery(_ref) {
    var context = _ref.context,
        resolve = _ref.resolve,
        reject = _ref.reject;

    var msg = context.options.channel.in_msg || "gql";
    var payload = (0, _networkInterface.printRequest)(context.request);
    chan.conn.push(msg, payload).receive("ok", resolve).receive("error", resolve).receive("timeout", reject.bind(null, 'timeout'));
  }

  return new Promise(function (resolve, reject) {
    if (!chan) {
      chan = socket.channels[channel.topic] = {
        conn: null,
        queue: []
      };
      socket.conn.onOpen(joinChannel.bind(null, resolve));
      socket.conn.onError(function (err) {
        if (chan.conn && (chan.conn.isJoined() || chan.conn.isJoining())) {
          chan.conn.leave();
        }
        chan.queue = [];
        resolve(err);
      });
    }
    if (chan.conn && chan.conn.isJoined()) {
      performQuery({ context: context, resolve: resolve, reject: reject });
    } else {
      chan.queue.push({ context: context, resolve: resolve, reject: reject });
      joinChannel(resolve);
    }
  });
}

var responseData = function responseData(_ref2) {
  var response = _ref2.response;

  return new Promise(function (resolve, reject) {
    if (response.error) {
      reject(response);
    } else if (response.data) {
      resolve(response);
    } else {
      reject('No response');
    }
  });
};

function createNetworkInterface(ifaceOpts) {
  var sockets = {};
  var middlewares = [];
  var afterwares = [];

  var use = (0, _ramda.map)(function (item) {
    return middlewares.push(item);
  });
  var useAfter = (0, _ramda.map)(function (item) {
    return afterwares.push(item);
  });

  var requestMiddleware = function requestMiddleware(request) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ request: request, options: options }, middlewares);
  };

  var responseMiddleware = function responseMiddleware(response) {
    var options = (0, _ramda.clone)(ifaceOpts);
    return applyWares({ response: response, options: options }, afterwares);
  };

  var query = (0, _ramda.pipeP)(requestMiddleware, executeQuery.bind(null, sockets), responseMiddleware, responseData);

  return { query: query, use: use, useAfter: useAfter };
}